import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.io.FileNotFoundException;
import java.io.File;

public class Board{
    private Piece[][] array;
    
    public Board(String fileName, Player white, Player black){
	int pieceType;
	int col;
	int row;
	String nextWord;
	Player owner;
	
	this.array = new Piece[8][8];
	try {
	    File file = new File(fileName);
    	    if(file.exists()==false) {
		System.err.println("Error: Cannot find file "+ fileName);
		System.exit(1);		
            } 

	    Scanner in = new Scanner(file);
	    while(in.hasNext()) {
		if ((nextWord = in.nextLine()).length()>2) {
		    pieceType = nextWord.charAt(0);
		    col = nextWord.charAt(1)-'0';
		    row = nextWord.charAt(2)-'0';
		    
		    owner = black;
		    if (pieceType >= 'a' && pieceType <= 'z')
    			owner = white;
		    switch(pieceType) {
		    case 'K' : case 'k' :  
    			{ this.addPiece(new King(col, row, owner)); break;}
		    case 'Q' : case 'q' :  
			{ this.addPiece(new Queen(col, row, owner)); break;}
		    case 'B' : case 'b' :  
			{ this.addPiece(new Bishop(col, row, owner)); break;}
		    case 'R' : case 'r' :  
			{ this.addPiece(new Rook(col, row, owner)); break;}
		    case 'N' : case 'n' :  
			{ this.addPiece(new Knight(col, row, owner)); break;}
			case 'P' : case 'p' :  
			{ this.addPiece(new Pawn(col, row, owner)); break;}
	    	}
	    	}	    	
	    }
	    in.close();
	}
	catch(FileNotFoundException e) {
	    System.err.println("Error file not found : "+e);
	    System.exit(1);	
	}
    }
    
    public List<Coordinates> getAllCoordinates(){
        List<Coordinates> allCoordinates = new ArrayList<>();
        for (int i = 0; i < 8; i++) for (int j = 0; j < 8; j++) allCoordinates.add(new Coordinates(j, i));
        return allCoordinates;
    }
    
    public List<Piece> getPieces(Player player) {
    	List<Piece> pieces = new ArrayList<>();
        List<Piece> all = getPieces();
        for (Piece piece : all) if (player.equals(piece.getOwner())) pieces.add(piece);
        return pieces;
    }

    public List<Piece> getPieces() {
        List<Piece> all = new ArrayList<>();
        for (Piece[] pieces : array) for (Piece piece : pieces) if (piece != null) all.add(piece);
        return all;
    }

    public void addPiece(Piece piece){
    	array[piece.getY()][piece.getX()] = piece;
    }

    public Piece getPiece(Coordinates coordinates){
    	return array[coordinates.getY()][coordinates.getX()];
    }

    public Piece getPiece(int x, int y){
    	return array[y][x];
    }

    public void emptyCell(Coordinates coordinates){
    	array[coordinates.getY()][coordinates.getX()] = null;
    }
    
    public boolean isEmptyCell(Coordinates coordinates){
    	return array[coordinates.getY()][coordinates.getX()] == null;
    }
    
    public boolean isEmptyCell(int x, int y){
    	return array[y][x] == null;
    }
    
    public boolean sameColumnNothingBetween(Coordinates origin, Coordinates destination){
        if (origin.equals(destination)) return false;
        if (origin.getX() != destination.getX()) return false;
        int min = Math.min(origin.getY(), destination.getY());
        int max = Math.max(origin.getY(), destination.getY());
        for (int i = min ; i <= max; i++) {
            if (!isEmptyCell(origin.getX(), i)) return false;
        }
        return true;
    }
    
    public boolean sameRowNothingBetween(Coordinates origin, Coordinates destination){
        if (origin.equals(destination)) return false;
        if (origin.getY() != destination.getY()) return false;
        int min = Math.min(origin.getX(), destination.getX());
        int max = Math.max(origin.getX(), destination.getX());
        for (int i = min + 1; i < max; i++) {
            if (!isEmptyCell(i, origin.getY())) return false;
        }
        return true;
    }

    public boolean sameDiagonalNothingBetween(Coordinates origin, Coordinates destination){
        if (origin.equals(destination)) return false;
        int lenX = Math.abs(origin.getX() - destination.getX());
        int lenY = Math.abs(origin.getY() - destination.getY());
        if (lenX != lenY) return false;
        if (origin.getX() - destination.getX() > 1) {
            if (origin.getY() - destination.getY() > 1)
                for (int i = 1; i < lenX; i++) if (!isEmptyCell(origin.getX() - i, origin.getY() - i)) return false;
            if (origin.getY() - destination.getY() < -1)
                for (int i = 1; i < lenX; i++) if (!isEmptyCell(origin.getX() - i, origin.getY() + i)) return false;
        }
        if (origin.getX() - destination.getX() < -1) {
            if (origin.getY() - destination.getY() > 1)
                for (int i = 1; i < lenX; i++) if (!isEmptyCell(origin.getX() + i, origin.getY() - i)) return false;
            if (origin.getY() - destination.getY() < -1)
                for (int i = 1; i < lenX; i++) if (!isEmptyCell(origin.getX() + i, origin.getY() + i)) return false;
        }
        return true;
    }
}
