import java.util.List;
import java.util.Stack;




public class GameUI {
    public Board board;
    private Player white;
    private Player black;
    private Player currentPlayer;
    private ChessUI ui;
    private Stack<Move> history;
    
    public GameUI(ChessUI ui, String boardConfigFileName, Player white, Player black){
        this.board = new Board(boardConfigFileName, white, black);
        this.white = white;
        this.black = black;
        this.currentPlayer = white;
        this.ui = ui;
	this.history = new Stack<Move>();

        for(Piece p : board.getPieces())
            this.ui.placePiece(p.getType(), p.getColor(), p.getPosition());
    }

    public Board getBoard(){
	return this.board;
    }

    public boolean undo(){
	if(this.history.empty()) return false;
	Move move = this.history.pop();
	board.emptyCell(move.destination);
	ui.removePiece(move.destination);
	if(move.pieceAtDestination != null){
	    move.pieceAtDestination.setPosition(move.destination);
	    board.addPiece(move.pieceAtDestination);
	    ui.placePiece(move.pieceAtDestination.getType(), move.pieceAtDestination.getColor(), move.pieceAtDestination.getPosition());
	}
	board.emptyCell(move.origin);
	ui.removePiece(move.origin);
	move.pieceAtOrigin.setPosition(move.origin);
	board.addPiece(move.pieceAtOrigin);
	ui.placePiece(move.pieceAtOrigin.getType(), move.pieceAtOrigin.getColor(), move.pieceAtOrigin.getPosition());
	
	currentPlayer = move.pieceAtOrigin.getOwner();
	if(move.pieceAtDestination != null)
	    currentPlayer.removeFromScore(move.pieceAtDestination.getValue());
	return true;
    }
    
    
    
    public boolean isMovePlayable(Move gameMove){
    	Piece piece = board.getPiece(gameMove.origin);
    	if(piece == null
    			||currentPlayer.getColor() != piece.getColor()
    			|| !gameMove.pieceAtOrigin.isMoveAuthorized(board, gameMove.destination)
    			 
    			|| gameMove.pieceAtOrigin.getOwner() != currentPlayer){
    		System.out.println("Le coups ne peut pas etre jouer 1");  
    		return false;}
    	
                
               

        if (gameMove.pieceAtDestination != null
                && gameMove.pieceAtDestination.sameColor(gameMove.pieceAtOrigin)) {
        	System.out.println("Le coups ne peut pas etre jouer 2");
        	
        	
        	
        	return false;}


        applyMove(gameMove);

        /**if (isCheck(getOpponent(currentPlayer))) {
            undo();
            return false;
        }

		***/
        undo();
        return true;
	
    }
    
    public void applyMove(Move move){
    	 Piece piece = board.getPiece(move.origin);
         history.push(move);
         board.emptyCell(piece.getPosition());
         piece.setPosition(move.destination);
         board.addPiece(piece);
         switchPlayers();
    }

    public void switchPlayers(){
    	this.currentPlayer = getOpponent(currentPlayer);
    }

    public Player getOpponent(Player player){
    	if (player == white) return black;
        return player;
    }

    public boolean isPrey(Piece prey){
        Player playerOp = getOpponent(prey.owner);
        List<Move> opponentMoves = playerOp.getAllMoves(board);
        for (Move move : opponentMoves) {
            if (prey.equals(move.pieceAtDestination)) return true;
        }
        return false;
    }

    public boolean isCheck(Player player){return isPrey(player.getKing());}

    public boolean isCheckMate(Player player){
	return false;
    }
    
    public void determineWinner(){
    }
    
    public void play(){
	int numberOfHits = 0;
	while(numberOfHits < 50){
		System.out.println("A toi de jouer " + currentPlayer.getColor());
		Move move;
        do {
            move = new Move(board, currentPlayer.getFromTo());
        } 
	    while(!isMovePlayable(move)); 
	    applyMove(move);
	    ui.placePiece(move.pieceAtOrigin.getType(), move.pieceAtOrigin.getColor(), move.destination);
        ui.removePiece(move.origin);
        numberOfHits++;
        switchPlayers();
	    
	    
	    
	    

	    numberOfHits ++;
	    System.out.println("Nombre de coups: " + numberOfHits+" /50");
	    switchPlayers();
	}
    }
}
