import java.awt.Color;

/**
 * Classe ByteGrayColor: Pour manipuler le coleur d'un pixel en niveau de gris (grey level).
 */

public class ByteGrayColor implements GrayColor {

    private static final int MINIMUM_GRAY_LEVEL = 0;
    private static final int MAXIMUM_GRAY_LEVEL = 255;
    private static final int OPACITY = 1;

    public static final ByteGrayColor WHITE = new ByteGrayColor(MAXIMUM_GRAY_LEVEL);
    public static final ByteGrayColor BLACK = new ByteGrayColor(MINIMUM_GRAY_LEVEL);

    private final int grayLevel;


    public ByteGrayColor(){
       this.grayLevel = MINIMUM_GRAY_LEVEL;
    }

    public ByteGrayColor(int grayLevel) {
       
        this.grayLevel = grayLevel;
    }

    public ByteGrayColor(double luminosity) {
    	double lum = MAXIMUM_GRAY_LEVEL * (int)luminosity;
        this.grayLevel =  (int)lum;
    }

    @Override
    public int getGrayLevel(){
    	return grayLevel;
    }

    @Override
    public double getLuminosity() {
        // TODO : Retourner la luminosité de la couleur (entre 0 noir et 1 blanc)
        return (double)this.grayLevel/MAXIMUM_GRAY_LEVEL;
    }

    @Override
    public Color getColor(){
        float component = (float)getLuminosity();
        return new Color(component, component, component);
    }


    @Override
    public int compareTo(GrayColor o) {
        // TODO : Retourner la différence de niveau de gris.
        return 0;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (this.getClass() != o.getClass()) return false;
        ByteGrayColor color = (ByteGrayColor) o;
        return this.compareTo(color) == 0;
    }

    @Override
    public GrayColor invert() {
        return new ByteGrayColor(MAXIMUM_GRAY_LEVEL - grayLevel);
    }

    public GrayColor decreaseGrayLevel(int nbGrayLevels){
        return new ByteGrayColor((Math.floor(grayLevel/255.0 * (nbGrayLevels-1)) * MAXIMUM_GRAY_LEVEL / (nbGrayLevels-1)));
    }



}
